// set the dimensions and margins of the graph
const width = 800;
const height = 500;

// append the svg object to the body of the page
const svg = d3
  .select('#my_dataviz')
  .append('svg')
  // default behavior of div is to put things side by side
  // if width were like 60%, it would see two things can't fit and put it below
  .attr('width', width / 2)
  .attr('height', height);

const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width / 2)
  .attr('height', height);

function render(mapData, countryData) {
  var projection = d3
    .geoEqualEarth()
    .fitSize([width / 2, height], mapData);
  // console.log(projection);

  // magic incantation
  var path = d3.geoPath(projection);

  // map projection isn't working because fitSize also needs latitude longitude
  //console.log(projection([60, 60]));
  const colorAttr = 'Birth Rate';
  const xDataRange = d3.extent(
    d3.map(
      countryData.values(),
      (v) => +v['Co2-Emissions'],
    ),
  );

  console.log(xDataRange);

  var colorScale = d3
    .scaleLinear()
    .domain(xDataRange)
    .range(['blue', 'red']);

  svg
    .append('g')
    .selectAll('path')
    .data(mapData.features)
    // d is the magic attribute for paths, like width and height for rectangles
    .join('path')
    .attr('d', path)
    // .attr('fill', (d) => console.log(d.properties.name))
    .attr('fill', (d) => {
      if (countryData.has(d.properties.name)) {
        return colorScale(
          countryData.get(d.properties.name)[colorAttr],
        );
      } else {
        return 'grey';
      }
    });
}
//   var projection = d3
//     .geoEqualEarth()
//     .fitSize([width / 2, height], mapData);
//   var path = d3.geoPath(projection);

//   const mapColorAttr = 'Birth Rate';
//   var mapColorScale = d3
//     .scaleLinear()
//     .domain(
//       d3.extent(
//         d3.map(
//           countryData.values(),
//           (r) => +r[mapColorAttr],
//         ),
//       ),
//     )
//     .range(['white', 'blue']);

//   function getCountryColor(name) {
//     let entry = countryData.get(name);
//     if (!(entry == undefined)) {
//       return mapColorScale(entry[mapColorAttr]);
//     }
//     return 'grey';
//   }

//   svg
//     .append('g')
//     .selectAll('path')
//     .data(mapData.features)
//     .join('path')
//     // draw each country
//     .attr('fill', function (d) {
//       return getCountryColor(d.properties.name);
//     })
//     .on('mouseover', function (event, data) {
//       //d3.select(this).attr('fill', 'red');
//       d3.selectAll(`#${this.id}`)
//         .attr('fill', 'red')
//         .attr('r', 20)
//         .selectAll('circle');
//     })
//     .on('mouseleave', function (event, data) {
//       d3.selectAll(`#${this.id}`)
//         .attr('fill', getCountryColor(data.properties.name))
//         .attr('r', (d) => rScale(d[rAttr]));
//     })
//     .attr('id', (d) => d.properties.name)
//     .attr('d', path);

//   const xAttr = 'Fertility Rate';
//   const x = d3
//     .scaleLinear()
//     .domain(
//       d3.extent(
//         d3.map(countryData.values(), (d) => d[xAttr]),
//       ),
//     )
//     .range([0, width / 2]);

//   const yAttr = 'Life expectancy';
//   const y = d3
//     .scaleLinear()
//     .domain(
//       d3.extent(
//         d3.map(
//           countryData.values().filter((d) => d[yAttr] > 0),
//           (d) => d[yAttr],
//         ),
//       ),
//     )
//     .range([height, 0]);

//   const plotColorAttr = 'Continent';
//   const plotColorScale = d3.scaleOrdinal(
//     d3.schemeCategory10,
//   );
//   function getPointColor(d) {
//     return plotColorScale(d[plotColorAttr]);
//   }

//   const rAttr = 'Population';
//   const rScale = d3
//     .scaleLinear()
//     .domain(
//       d3.extent(
//         d3.map(countryData.values(), (d) => d[rAttr]),
//       ),
//     )
//     .range([4, 15]);

//   svg2
//     .append('g')
//     .attr('transform', `translate(0,${height - 50})`) // This controls the vertical position of the Axis
//     .call(d3.axisBottom(x));

//   svg2
//     .append('g')
//     .attr('transform', `translate(0,0)`) // This controls the vertical position of the Axis
//     .call(d3.axisRight(y));

//   svg2
//     .append('g')
//     .selectAll('circle')
//     .data(countryData.values().filter((c) => +c[yAttr] > 0))
//     .join('circle')
//     .attr('cx', (d) => x(d[xAttr]))
//     .attr('cy', (d) => y(d[yAttr]))
//     .attr('r', (d) => rScale(d[rAttr]))
//     .attr('fill', getPointColor)
//     .attr('id', (d) => d.Country)
//     .on('mouseover', function (event, data) {
//       //d3.select(this).attr('fill', 'red');
//       d3.selectAll(`#${this.id}`)
//         .attr('fill', 'red')
//         .attr('r', 20)
//         .selectAll('circle');
//     })
//     .on('mouseleave', function (event, data) {
//       d3.selectAll(`#${this.id}`)
//         .attr('fill', getCountryColor(data.Country))
//         .attr('r', (d) => rScale(d[rAttr]));
//     });
// }

//Read the data
// world country geojson data
const mapUrl =
  'https://raw.githubusercontent.com/holtzy/D3-graph-gallery/master/DATA/world.geojson';
// country data
const countryStatsUrl =
  'https://raw.githubusercontent.com/bepnye/CP341/main/world-data-2023.csv';
// continent data which connects each country to a continent so you can color by continent
const continentUrl =
  'https://raw.githubusercontent.com/samayo/country-json/master/src/country-by-continent.json';

function normalizeCountryName(s) {
  return s.replaceAll(' ', '_').toLowerCase();
}

// once all three promises get resolved, use the then handler to make an array
// should inspect the data loading in the console to check it loaded and see how its formatted
Promise.all([
  d3.json(mapUrl),
  d3.csv(countryStatsUrl),
  d3.json(continentUrl),
]).then(([mapData, csvData, continentData]) => {
  // var countryData = new Map();
  // csvData.forEach((row) => {
  //   row.Country = normalizeCountryName(row.Country);
  //   row.Population = +row.Population.replaceAll(',', '');
  //   row.Continent = 'Unknown';
  //   countryData.set(row.Country, row);
  // });
  // mapData.features.forEach((f) => {
  //   f.properties.name = normalizeCountryName(
  //     f.properties.name,
  //   );
  // });
  // continentData.forEach((row) => {
  //   let name = normalizeCountryName(row.country);
  //   if (countryData.has(name)) {
  //     countryData.get(name).Continent = row.continent;
  //   } else {
  //     console.log('No match found for ', name);
  //   }
  // });

  // console.log(mapData);
  // console.log(csvData);
  // console.log(continentData);

  var countryData = new Map();
  csvData.forEach((row) => {
    // console.log(row);
    row.Population = row.Population.replaceAll(',', '');
    row['Co2-Emissions'] = row['Co2-Emissions'].replaceAll(
      ',',
      '',
    );
    countryData.set(row.Country, row);
  });
  // console.log(countryData);
  render(mapData, countryData);
});
